# -*- coding: utf-8  -*-
from __future__ import division

import json
import re

import numpy as np

FILENAME = 'all_trump_tweets.json'


def read_file(file_name):
    with open(file_name, 'r') as f:
        records = json.load(f)
    # output ist eine Liste von Dictionaries
    return records


def extract_text(records):
    """Extrahiere Text aus den Tweets und bereinige von Wörtern wie Twitter-Accountnamen.
    :param records: Liste von Tweets (list of strings)
    :return: Liste von Listen von Wörtern (list of list of strings)
    """

    def correct(w):
        """Gib True zurück, wenn keine der Bedingungen in der letzten Zeile erfüllt ist, sonst False
        """
        p = re.compile('.*\d')  # beliebiger string, der mit einer Ziffer endet
        return not (w.startswith('@') or p.match(w) or w.find('http') > -1)

    texts = []
    chars_to_replace = ['.', ',', '!', '?', ';', ':', '"', "'", u'\u201e', u'\u201c', u'\u2019', u'\u2026', ')', '(',
                        u'\xab', u'\xbb']
    for r in records:
        text = r['text'].lower()
        for c in chars_to_replace:
            text = text.replace(c, ' ')
        words = text.split()
        # reduziert die Liste 'words' auf die Einträge, für welche die Funktion 'correct' den Wert 'True' zurückgibt
        words_correct = filter(correct, words)
        texts.append(words_correct)

    return texts


def extract_words(texts):
    """Extrahiere alle Wörter aus der Liste von Listen von Wörtern und speichere sie in einer flachen Liste"""
    all_words = []
    for l in texts:
        all_words.extend(l)

    return all_words


def word_frequency(words):
    """Berechne relative Häufigkeiten von Wörtern"""
    frq = {}
    for word in words:
        if word in frq:
            frq[word] += 1
        else:
            frq[word] = 1

    summe = 0
    for k in frq.keys():
        summe += frq[k]
    for l in frq.keys():
        frq[l] = frq[l] / summe  # berechne relative Häufigkeiten

    return frq


def compute_rel_freq_of_word_pairs(words):
    """Berechne die relativen Häufigkeiten von Paaren von Wörtern in 'words' (nach der Berechnung von absoluten)."""
    bigram = {}
    unique_words = list(set(words))  # extrahiere die Wörter ohne Mehrfachvorkommen
    for word in unique_words:
        bigram[word] = {}

    for i, word in enumerate(words[:-1]):
        nextWord = words[i + 1]
        if nextWord in bigram[word].keys():
            bigram[word][nextWord] += 1
        else:
            bigram[word][nextWord] = 1

    for word in bigram.keys():
        abs_word_occurence = sum([bigram[word][nachfolger] for nachfolger in bigram[word]])

        for i in bigram[word]:
            bigram[word][i] = bigram[word][i] / abs_word_occurence

    return bigram


def tweet_generator(word_freq, bigram_freq):
    """Generiere Tweets"""
    while True:
        wort = np.random.choice(word_freq.keys(), p=[word_freq[q] for q in word_freq.keys()])
        if len(wort) > 1:
            break
    while True:
        print wort + " ",
        wort = np.random.choice(bigram_freq[wort].keys(), p=[bigram_freq[wort][q] for q in bigram_freq[wort].keys()])
        if wort == '//t':
            break

if __name__ == '__main__':
    tweet_records = read_file(FILENAME)  # tweet_records ist eine Liste von dictionaries mit Tweets
    tweets = extract_text(tweet_records)  # tweets ist eine geschachtelte Liste mit Wörtern aus den Tweets
    all_tweet_words = extract_words(tweets)  # all_tweet_words ist eine flache Liste von Tweet-Wörtern; Wörter kommen mehrfach vor
    rel_word_freq = word_frequency(
        all_tweet_words)  # rel_word_freq ist ein dictionary: Wörter als keys und rel. Häufigkeiten als values
    rel_bigram_freq = compute_rel_freq_of_word_pairs(all_tweet_words)
    tweet_generator(rel_word_freq, rel_bigram_freq)
